@@ -111,7 +111,6 @@ struct AppCreator {
 
 struct AppCreatorOutput {
     app: AppConfigV1,
-    pkg: StringWebcIdent,
     api_pkg: Option<Package>,
     local_package: Option<(PathBuf, wasmer_toml::Manifest)>,
 }
@@ -138,12 +137,6 @@ impl AppCreator {
         let outer_pkg_name =
             crate::utils::prompts::prompt_for_ident("Package name", Some(&default_name))?;
         let outer_pkg_full_name = format!("{}/{}", self.owner, outer_pkg_name);
-        let outer_pkg = StringWebcIdent(edge_schema::schema::WebcIdent {
-            repository: None,
-            namespace: self.owner.clone(),
-            name: outer_pkg_name.clone(),
-            tag: None,
-        });
 
         eprintln!("What should be the name of the app?");
 
@@ -199,9 +192,12 @@ impl AppCreator {
         let app_cfg = AppConfigV1 {
             app_id: None,
             name: app_name,
+            owner: Some(self.owner.clone()),
             cli_args: None,
             env: Default::default(),
             volumes: None,
+            domains: None,
+            scaling: None,
             package: edge_schema::schema::StringWebcIdent(edge_schema::schema::WebcIdent {
                 repository: None,
                 namespace: self.owner,
@@ -211,11 +207,11 @@ impl AppCreator {
             capabilities: None,
             scheduled_tasks: None,
             debug: Some(false),
+            extra: Default::default(),
         };
 
         Ok(AppCreatorOutput {
             app: app_cfg,
-            pkg: outer_pkg,
             api_pkg: None,
             local_package: Some((self.dir, manifest)),
         })
@@ -336,9 +332,11 @@ impl AppCreator {
         // TODO: check if name already exists.
         let cfg = AppConfigV1 {
             app_id: None,
+            owner: Some(self.owner.clone()),
             volumes: None,
             name,
             env: Default::default(),
+            scaling: None,
             // CLI args are only set for JS and Py workers for now.
             cli_args,
             // TODO: allow setting the description.
@@ -347,12 +345,13 @@ impl AppCreator {
             capabilities: None,
             scheduled_tasks: None,
             debug: Some(false),
+            domains: None,
+            extra: Default::default(),
         };
 
         Ok(AppCreatorOutput {
             app: cfg,
             api_pkg,
-            pkg,
             local_package,
         })
     }
@@ -482,7 +481,7 @@ impl AsyncCliCommand for CmdAppCreate {
             type_,
             interactive,
             dir: base_dir,
-            owner,
+            owner: owner.clone(),
             api,
             user,
             local_package,
@@ -499,8 +498,8 @@ impl AsyncCliCommand for CmdAppCreate {
         let AppCreatorOutput {
             app: cfg,
             api_pkg,
-            pkg,
             local_package,
+            ..
         } = output;
 
         let deploy_now = if self.offline {
@@ -551,7 +550,7 @@ impl AsyncCliCommand for CmdAppCreate {
                 original_config: None,
                 allow_create: true,
                 make_default: true,
-                owner: Some(pkg.0.namespace.clone()),
+                owner: Some(owner.clone()),
                 wait: wait_mode,
             };
             let (_app, app_version) = deploy_app_verbose(&api, opts).await?;
@@ -604,6 +603,7 @@ mod tests {
             r#"---
 kind: wasmer.io/App.v0
 name: static-site-1
+owner: testuser
 package: testuser/static-site-1@0.1.0
 debug: false
 "#,
@@ -637,6 +637,7 @@ debug: false
             r#"---
 kind: wasmer.io/App.v0
 name: testapp
+owner: wasmer
 package: wasmer/testpkg
 debug: false
 "#,
@@ -669,6 +670,7 @@ debug: false
             r#"---
 kind: wasmer.io/App.v0
 name: test-js-worker
+owner: wasmer
 package: wasmer/test-js-worker
 cli_args:
   - /src/index.js
@@ -704,6 +706,7 @@ debug: false
             r#"---
 kind: wasmer.io/App.v0
 name: test-py-worker
+owner: wasmer
 package: wasmer/test-py-worker
 cli_args:
   - /src/main.py
