@@ -35,6 +35,7 @@ use wasmer_wasix::{
     journal::CompactingLogFileJournal,
     runners::{
         dcgi::{DcgiInstanceFactory, DcgiRunner},
+        dproxy::DProxyRunner,
         emscripten::EmscriptenRunner,
         wasi::WasiRunner,
         wcgi::{self, AbortHandle, NoOpWcgiCallbacks, WcgiRunner},
@@ -185,6 +186,8 @@ impl Run {
 
         if DcgiRunner::can_run_command(cmd.metadata())? {
             self.run_dcgi(id, pkg, uses, runtime)
+        } else if DProxyRunner::can_run_command(cmd.metadata())? {
+            self.run_dproxy(id, pkg, runtime)
         } else if WcgiRunner::can_run_command(cmd.metadata())? {
             self.run_wcgi(id, pkg, uses, runtime)
         } else if WasiRunner::can_run_command(cmd.metadata())? {
@@ -302,6 +305,17 @@ impl Run {
         runner.run_command(command_name, pkg, runtime)
     }
 
+    fn run_dproxy(
+        &self,
+        command_name: &str,
+        pkg: &BinaryPackage,
+        runtime: Arc<dyn Runtime + Send + Sync>,
+    ) -> Result<(), Error> {
+        let mut inner = self.build_wasi_runner(&runtime)?;
+        let mut runner = wasmer_wasix::runners::dproxy::DProxyRunner::new(inner, pkg);
+        runner.run_command(command_name, pkg, runtime)
+    }
+
     fn run_emscripten(
         &self,
         command_name: &str,
