@@ -3,7 +3,7 @@ use std::ops::Deref;
 use super::*;
 
 impl ReadableJournal for Box<DynReadableJournal> {
-    fn read(&self) -> anyhow::Result<Option<JournalEntry<'_>>> {
+    fn read(&self) -> anyhow::Result<Option<LogReadResult<'_>>> {
         self.deref().read()
     }
 
@@ -13,13 +13,13 @@ impl ReadableJournal for Box<DynReadableJournal> {
 }
 
 impl WritableJournal for Box<DynWritableJournal> {
-    fn write<'a>(&'a self, entry: JournalEntry<'a>) -> anyhow::Result<u64> {
+    fn write<'a>(&'a self, entry: JournalEntry<'a>) -> anyhow::Result<LogWriteResult> {
         self.deref().write(entry)
     }
 }
 
 impl ReadableJournal for Box<DynJournal> {
-    fn read(&self) -> anyhow::Result<Option<JournalEntry<'_>>> {
+    fn read(&self) -> anyhow::Result<Option<LogReadResult<'_>>> {
         self.deref().read()
     }
 
@@ -29,7 +29,7 @@ impl ReadableJournal for Box<DynJournal> {
 }
 
 impl WritableJournal for Box<DynJournal> {
-    fn write<'a>(&'a self, entry: JournalEntry<'a>) -> anyhow::Result<u64> {
+    fn write<'a>(&'a self, entry: JournalEntry<'a>) -> anyhow::Result<LogWriteResult> {
         self.deref().write(entry)
     }
 }
