@@ -9,7 +9,7 @@ pub struct NullJournal {
 }
 
 impl ReadableJournal for NullJournal {
-    fn read(&self) -> anyhow::Result<Option<JournalEntry<'_>>> {
+    fn read(&self) -> anyhow::Result<Option<LogReadResult<'_>>> {
         Ok(None)
     }
 
@@ -19,11 +19,14 @@ impl ReadableJournal for NullJournal {
 }
 
 impl WritableJournal for NullJournal {
-    fn write<'a>(&'a self, entry: JournalEntry<'a>) -> anyhow::Result<u64> {
+    fn write<'a>(&'a self, entry: JournalEntry<'a>) -> anyhow::Result<LogWriteResult> {
         if self.debug_print {
             tracing::debug!("journal event: {:?}", entry);
         }
-        Ok(entry.estimate_size() as u64)
+        Ok(LogWriteResult {
+            record_start: 0,
+            record_end: entry.estimate_size() as u64,
+        })
     }
 }
 
