@@ -49,8 +49,8 @@ impl VirtualFile for DualWriteFile {
         self.inner.set_len(new_size)
     }
 
-    fn unlink(&mut self) -> BoxFuture<'static, Result<()>> {
-        Box::pin(self.inner.unlink())
+    fn unlink(&mut self) -> Result<()> {
+        self.inner.unlink()
     }
 
     fn poll_read_ready(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<io::Result<usize>> {
