@@ -442,9 +442,8 @@ impl VirtualFile for File {
         fs::File::set_len(&self.inner_std, new_size).map_err(Into::into)
     }
 
-    fn unlink(&mut self) -> BoxFuture<'static, Result<()>> {
-        let path = self.host_path.clone();
-        Box::pin(async move { fs::remove_file(&path).map_err(Into::into) })
+    fn unlink(&mut self) -> Result<()> {
+        fs::remove_file(&self.host_path).map_err(Into::into)
     }
 
     fn get_special_fd(&self) -> Option<u32> {
@@ -589,8 +588,8 @@ impl VirtualFile for Stdout {
         Ok(())
     }
 
-    fn unlink(&mut self) -> BoxFuture<'static, Result<()>> {
-        Box::pin(async { Ok(()) })
+    fn unlink(&mut self) -> Result<()> {
+        Ok(())
     }
 
     fn get_special_fd(&self) -> Option<u32> {
@@ -779,8 +778,8 @@ impl VirtualFile for Stderr {
         Ok(())
     }
 
-    fn unlink(&mut self) -> BoxFuture<'static, Result<()>> {
-        Box::pin(async { Ok(()) })
+    fn unlink(&mut self) -> Result<()> {
+        Ok(())
     }
 
     fn get_special_fd(&self) -> Option<u32> {
@@ -915,8 +914,8 @@ impl VirtualFile for Stdin {
     fn set_len(&mut self, _new_size: u64) -> crate::Result<()> {
         Ok(())
     }
-    fn unlink(&mut self) -> BoxFuture<'static, Result<()>> {
-        Box::pin(async { Ok(()) })
+    fn unlink(&mut self) -> Result<()> {
+        Ok(())
     }
     fn get_special_fd(&self) -> Option<u32> {
         Some(0)
