@@ -207,8 +207,8 @@ impl VirtualFile for File {
         Err(FsError::PermissionDenied)
     }
 
-    fn unlink(&mut self) -> BoxFuture<'static, crate::Result<()>> {
-        Box::pin(async { Err(FsError::PermissionDenied) })
+    fn unlink(&mut self) -> crate::Result<()> {
+        Err(FsError::PermissionDenied)
     }
 
     fn poll_read_ready(
